#include "MotionDetection.h"
#include <math.h>

MotionDetection::MotionDetection(){
    handler = new SPIClass(FSPI);
};

void MotionDetection::begin(void){
    pinMode(34,OUTPUT);
    digitalWrite(34,HIGH);
    handler->begin(36,37,35,34);
    // set Accel and Gyroscop to Low Noise
    this->writeRegister(PWR_MGMT0,0x1F);
    //busy Wait for startup 
    delayMicroseconds(250);
    //set accelconfig
    this->writeRegister(0x21,0x05);
    //set Gyroconfig
    this->writeRegister(0x20,0x25);
    //set Gyro Filter
    this->writeRegister(0x23,0x37);
    //Enable Gyro and Acceldata in FIFO
    this->initFIFO();
};
void MotionDetection::end(void){
    this->writeRegister(PWR_MGMT0,0x00);
};
IMUResult MotionDetection::getAcceleration(){
    IMUResult result;
    result.x = readRegister(ACCEL_DATA_X_HIGH)<<8;
    result.x |= readRegister(ACCEL_DATA_X_LOW);
    result.y = readRegister(ACCEL_DATA_Y_HIGH)<<8;
    result.y |= readRegister(ACCEL_DATA_Y_LOW);
    result.z = readRegister(ACCEL_DATA_Z_HIGH)<<8;
    result.z |= readRegister(ACCEL_DATA_Z_LOW);
    return result;
};
IMUResult MotionDetection::getRotation(){
    IMUResult result;
    result.x = readRegister(GYRO_DATA_X_HIGH) <<8;
    result.x |= readRegister(GYRO_DATA_X_LOW);
    result.y = readRegister(GYRO_DATA_Y_HIGH)<<8;
    result.y |= readRegister(GYRO_DATA_Y_LOW);
    result.z = readRegister(GYRO_DATA_Z_HIGH)<<8;
    result.z |= readRegister(GYRO_DATA_Z_LOW);
    return result;
};
float MotionDetection::getTemperature(){
    int16_t raw_temperatur = readRegister(REG_TEMP_HIGH)<<8;
    raw_temperatur |= readRegister(REG_TEMP_LOW); 
    return raw_temperatur/128 +25;
};

int8_t MotionDetection::getWhoAmI(){
    return readRegister(WHO_AM_I);    
};

uint8_t MotionDetection::cmdRead(uint8_t reg){
    return (CMD_READ | (reg & ADDR_MASK));
};
uint8_t MotionDetection::cmdWrite(uint8_t reg){
    return (CMD_WRITE | (reg & ADDR_MASK));
};

uint8_t MotionDetection::readRegister(uint8_t reg){
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    uint8_t result;
    result = handler->transfer(cmdRead(reg));
    result = handler->transfer(0x00);
    digitalWrite(34,HIGH);
    handler->endTransaction(); 
    return result;
};

uint8_t MotionDetection::readFromRegisterBank(registerBank bank,uint8_t reg){
    uint8_t result = 0;
    switch(bank){
        case(MREG1):
            this->writeRegister(BLK_SEL_R,0x00);
            break;
        case(MREG2):
            this->writeRegister(BLK_SEL_R,0x28);
            break;
        case(MREG3):
            this->writeRegister(BLK_SEL_R,0x50);
            break;
    }
    this->writeRegister(MADDR_R,reg);
    delayMicroseconds(10);
    result=this->readRegister(M_R);
    delayMicroseconds(10);
    this->resetRegisterBankAccess();
    return result;
};

void MotionDetection::writeToRegisterBank(registerBank bank, uint8_t reg, uint8_t value){
    while((this->readRegister(MCLK_RDY))&0x08!=0x08){
        Serial.println("CLK not rdy");
        delay(100);
    }
    uint8_t result = this->readRegister(PWR_MGMT0);
    Serial.print("MADDR_W: ");
    Serial.println(readRegister(MADDR_W)); 
    //set Idle Bit
    this->writeRegister(PWR_MGMT0,result|0x10);
    switch(bank){
        case(MREG1):
           this->writeRegister(BLK_SEL_W,0x00);
            break;
        case(MREG2):
            this->writeRegister(BLK_SEL_W,0x28);
            break;
        case(MREG3):
            this->writeRegister(BLK_SEL_W,0x50);
            break;
    }
    this->writeRegister(MADDR_W,reg);
    this->writeRegister(M_W,value);
    delayMicroseconds(10);
    this->writeRegister(PWR_MGMT0,result&0xEF);
    Serial.print("MADDR_W: ");
    Serial.println(readRegister(MADDR_W));
    this->resetRegisterBankAccess();
};

void MotionDetection::resetRegisterBankAccess(){
    this->writeRegister(BLK_SEL_R,0x00);
    this->writeRegister(BLK_SEL_W,0x00);
    this->writeRegister(MADDR_R,0x00);
    this->writeRegister(MADDR_W,0x00);
};

void MotionDetection::testFIFO(){
    uint16_t fifocount;
    
        fifocount = 0;
        fifocount = (this->readRegister(FIFO_COUNTH)<<8);
        fifocount |= this->readRegister(FIFO_COUNTL);
        Serial.print("FiFo Count: ");
        Serial.println(fifocount);
    
    Serial.print("FiFoData: ");
     handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    handler->transfer(cmdRead(0x3f));
    handler->transfer(buf,8*fifocount);
    //delayMicroseconds(10);
    digitalWrite(34,HIGH);
    handler->endTransaction();
    for(int i=0;i<fifocount;i++){

       /* Serial.println(buf[8*i]);
        Serial.println((buf[0x02+8*i]<<8)|buf[0x01+8*i]);
        Serial.println((buf[0x04+8*i]<<8)|buf[0x03+8*i]);
        Serial.println((buf[0x06+8*i]<<8)|buf[0x05+8*i]);
        Serial.println("=============");*/
    }
    
    writeRegister(0x02,0x04);
    delayMicroseconds(10);
    fifocount = 0;
    
        fifocount = (this->readRegister(FIFO_COUNTH)<<8);
        fifocount |= this->readRegister(FIFO_COUNTL);
        Serial.print("FiFo Count: ");
        Serial.println(fifocount);
}

void MotionDetection::initFIFO(){
    delay(60);
    //set INTF_CONFIG0 FIFO_COUNT_REC_RECORD und Little Endian
    this->writeRegister(INTF_CONFIG0,0x60);
    //set FIFO_CONFIG1 to Mode Snapshot and BYPASS Off
    this->writeRegister(FIFO_CONFIG1,0x00);
    //set TMST_CONFIG1_MREG1 TMST_CONFIIG1_TMST_EN
    this->writeToRegisterBank(MREG1,TMST_CONFIG1,0x00);
    //set FiFO config 5 GYRO_EN,TMST_FSYNC, ACCEL_EN, WM_GT_TH_EN
    this->writeToRegisterBank(MREG1,FIFO_CONFIG5,0x23);
    //set FOF_CONFIG2 0x1 (INT triggerd each packaged)
    this->writeRegister(FIFO_CONFIG2,0x0A);
};

uint MotionDetection::getDataFromFIFO(FIFO_Package* buffer){
    int16_t fifocount = 0;
    int8_t fifohigh = this->readRegister(FIFO_COUNTH);
    int8_t fifolow = this->readRegister(FIFO_COUNTL);  
    fifocount = (fifohigh<<8)|fifolow;
    //fifocount |= this->readRegister(FIFO_COUNTL);
    //fifocount = (this->readRegister(FIFO_COUNTH)<<8);
    Serial.println(fifolow);
    Serial.println(fifohigh);
    Serial.println(fifocount);
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    handler->transfer(cmdRead(FIFO_DATA));
    handler->transfer(buf,16*fifocount);
    digitalWrite(34,HIGH);
    handler->endTransaction();
    
    writeRegister(0x02,0x04);
    delayMicroseconds(10);

    for(int i = 0; i<fifocount;i++){
        buffer[i].header = buf[0x00+16*i];
        buffer[i].accel.x = (buf[0x02+16*i]<<8)|buf[0x01+16*i]; 
        buffer[i].accel.y = (buf[0x04+16*i]<<8)|buf[0x03+16*i]; 
        buffer[i].accel.z = (buf[0x06+16*i]<<8)|buf[0x05+16*i]; 
        buffer[i].gyro.x = (buf[0x08+16*i]<<8)|buf[0x07+16*i]; 
        buffer[i].gyro.y = (buf[0x0A+16*i]<<8)|buf[0x09+16*i]; 
        buffer[i].gyro.z = (buf[0x0C+16*i]<<8)|buf[0x0B+16*i]; 
        buffer[i].temperature = buf[0x0D+16*i]; 
        buffer[i].timestamp = (buf[0x0F+16*i]<<8)|buf[0x0E +16*i]; 
    }
    return fifocount;
};

void MotionDetection::writeRegister(uint8_t reg, uint8_t value){
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    handler->transfer(reg);
    handler->transfer(value);
    digitalWrite(34,HIGH);
    delayMicroseconds(10);
    handler->endTransaction();
};